

; DIGITAL-OSZILLOSKOP												
; 20 MHZ-Takt														
; AUTOR: Bernhard "Bernhard.Erfurt@gmx.de"							
; AUGUST 2005														
;																	
; Programm-Ablauf:													
; 1. Der ADC wird gestartet und jeder 8 Bit-Messwert				
;    wird in den SRAM geschrieben (bei ATmega8 ca. 800)				
; 2. Anschlieend werden alle Messwerte ber USART ausgegebnen		
; 3. nach einer kurzen Pause, denn der PC muss die Daten 			
;    auch grafisch darstellen knnen, beeginnt alles wieder von vorn

; Port-Belegung					
; PB0 ==>
; PB1 ==>
; PB2 ==>
; PB3 ==>
; PB6 ==>	Takteingang (20 MHz)
; PB7 ==>

; PC0 ==>
; PC1 ==>
; PC2 ==>
; PC3 ==>
; PC4 ==>
; PC5 ==> 	Eingang OSZILLOSKOP	

; PD0 ==>
; PD1 ==>	AUSGANG (TXD)			
; PD2 ==>
; PD3 ==>
; PD4 ==>
; PD5 ==>
; PD6 ==>
; PD7 ==>

; SRAM -Belegung										
;														
; 0x0000-005F   	intern genutzt						
;														
; 0x0060+100		DATEN								
				  	    
.include "m8def.inc"

.equ CLOCK 	= 20000000			; TAKT
.equ BAUD 	= 56000				;  Baudrate 56000; 19200; 9600 ;4800; 2400 ; 1200 
.equ UBRRVAL= CLOCK/(BAUD*16)-1 ; Berechnung


.equ adr_DATEN					= 0x0060	; START Adresse aufbereiteter Daten
.equ adr_DATEN_SPEICHER			= 0x0060+100

.equ ANZAHL_DATENPUNKTE		= 800 ; ANZAHL DATENPUNKTE max 800

.def ENDE_SPEICHER	=r10		; Ende Speicher erreicht? 0=nein 1=ja	
.def ADC_RUN		=r11		; ADC-RUN?				  0=nein 1=ja	

.def temp  = r16
.def temp1 = r17
.def temp2 = r18
.def temp3 = r19


.cseg					;Beginn eines Code-Segmentes
.org 0					;Startadresse=0				

rjmp RESET      ; Reset Handler                           
reti ;rjmp EXT_INT0   ; IRQ0 Handler                      
reti ;rjmp EXT_INT1   ; IRQ1 Handler                      
reti ;rjmp TIM2_COMP  ; Timer2 Compare Handler            
reti ;rjmp TIM2_OVF   ; Timer2 Overflow Handler           
reti ;rjmp TIM1_CAPT  ; Timer1 Capture Handler            
reti ;rjmp TIM1_COMPA ; Timer1 CompareA Handler           
reti ;rjmp TIM1_COMPB ; Timer1 CompareB Handler           
reti ; Timer1 Overflow Handler           	  			  
reti ;rjmp TIM0_OVF   ; Timer0 Overflow Handler           
reti ;rjmp SPI_STC    ; SPI Transfer Complete Handler     
reti ;rjmp USART_RXC  ; USART RX Complete Handler         
reti ;rjmp USART_UDRE ; UDR Empty Handler                 
reti ;rjmp USART_TXC  ; USART TX Complete Handler         
rjmp ADC_HANDLER   	  ; ADC Conversion Complete Handler   
reti ;rjmp EE_RDY     ; EEPROM Ready Handler              
reti ;rjmp ANA_COMP   ; Analog Comparator Handler         
reti ;rjmp TWSI       ; Two-wire Serial Interface Handler 
reti ;rjmp SPM_RDY    ; Store Program Memory Ready Handler


RESET:                        

; STACK initialisieren
ldi temp, LOW(RAMEND)      ; LOW-Byte der obersten RAM-Adresse
out SPL, temp
ldi temp, HIGH(RAMEND)     ; HIGH-Byte der obersten RAM-Adresse
out SPH, temp

;PORT B setzen
ldi temp, 0b00000000    ;Port B = Eingang
out DDRB, temp
;PORT C setzen
ldi temp, 0b00000000    ;Port C = Eingang
out DDRC, temp
;PORT D setzen
ldi temp, 0b00000010    ;Port D = Ausgang/Eingang
out DDRD, temp

; ADC initialisieren  			
; REFS1 - externe Referenz    	
; REFS0 - externe Referenz    	
; ADLAR - left Adjust Result  	
; MUX3  - KANAL-Wahl          	
; MUX2  - KANAL-Wahl          	
; MUX1  - KANAL-Wahl          	
; MUX0  - KANAL-Wahl          	
ldi temp, (0<<REFS1)|(0<<REFS0)|(0<<ADLAR)|(0<<MUX3)|(1<<MUX2)|(0<<MUX1)|(1<<MUX0)
out ADMUX, temp

; Baudrate einstellen				
ldi temp, LOW(UBRRVAL)
out UBRRL, temp
ldi temp, HIGH(UBRRVAL)
out UBRRH, temp
; Frame-Format: 8 Bit				
ldi temp, (1<<URSEL)|(3<<UCSZ0)
out UCSRC, temp
; TX aktivieren						
sbi UCSRB,TXEN                    


; SIMULATION 						
ldi XH,high(adr_daten_SPEICHER)		; X-REGISTER  Initialisierung
ldi XL, low(adr_daten_SPEICHER)		; X-REGISTER  Initialisierung
clr YL								; Y-ZHLER_RESET (Anzahl WERTE)
clr YH								; Y-ZHLER_RESET (Anzahl WERTE)
ldi ZH, high(TEXT_SIMULATION_2*2)	; SIMULATION-WERTE		
ldi ZL,  low(TEXT_SIMULATION_2*2)	; SIMULATION-WERTE		
SIMULATION:
	lpm temp, Z+   			; aus Tabelle laden und Zeiger um 1 erhhen	
	tst temp  				; TEMP = 0 ?								
	breq SIMULATION_e 		; ja => Sprung								
	st X+, temp				; schreiben und um eins erhhen				
	adiw YL,1				; Anzahl der Werte bestimmen				
	rjmp SIMULATION
SIMULATION_e:
;-----------------------------------------------------------------------
rcall BERECHNUNG_GLEICHSPANNUNGSANTEIL
rcall AUSGABE_SRAM_WERTE

;STARTWERTE	
clr ADC_RUN

;Interrupts freigeben
sei

;######################################################################## 
;######################################################################## 
;######################################################################## 
loop:
	tst ADC_RUN		; ist ADC schon aktiv ?	
	brne loop		; nein => Sprung		
	rcall wait		; kurze Pause			
	rcall ADC_START	; START ADC				
rjmp loop
;####################################################################################
;####################################################################################
;####################################################################################
ADC_START:
; ADC initialisieren									
; Bit 7 -ADEN: ADC Enable                             	
; Bit 6 -ADSC: ADC Start Conversion                   	
; Bit 5 -ADFR: ADC Free Running Select                	
; Bit 4 -ADIF: ADC Interrupt Flag                     	
; Bit 3 -ADIE: ADC Interrupt Enable                   	
; Bit 2 -ADPS2: Vorteiler                             	
; Bit 1 -ADPS2: Vorteiler                             	
; Bit 0 -ADPS2: Vorteiler                             	
; Vorteiler von 2,4,8,16,32,64,128 einstellbar        	
;	000=0 =>   2 x 13 Takte	=   26 Takte				
;	001=1 =>   2 x 13 Takte	=   26 Takte				
;	010=2 =>   4 x 13 Takte	=   52 Takte				
;  *011=3 =>   8 x 13 Takte	=  104 Takte				
;	100=4 =>  16 x 13 Takte	=  208 Takte				
;	101=5 =>  32 x 13 Takte	=  416 Takte				
;	110=6 =>  64 x 13 Takte	=  832 Takte				
;	111=7 => 128 x 13 Takte	= 1664 Takte				
ldi temp, (1<<ADEN)|(1<<ADSC)|(1<<ADFR)|(1<<ADIF)|(1<<ADIE)  + 3
out ADCSRA, temp
; 
	ldi XL, low(adr_daten_SPEICHER)		; X-DATEN-Initialisierung		
	ldi XH,high(adr_daten_SPEICHER)		; X-DATEN-Initialisierung		
	clr ZL								; Z-ZHLER-RESET (Anzahl WERTE)	
	clr ZH								; Z-ZHLER-RESET (Anzahl WERTE)	
	clr ENDE_SPEICHER					; Ende erreicht 0=nein 1=ja		
	ldi temp,1
	mov ADC_RUN, temp
ret
;####################################################################################
;####################################################################################
;####################################################################################
ADC_HANDLER: 
	push temp           ; Temp  auf dem Stack sichern            
	push temp1          ; Temp1 auf dem Stack sichern            
	push temp2          ; Temp2 auf dem Stack sichern            
	push temp3          ; Temp3 auf dem Stack sichern            
	in TEMP,SREG		; Einlesen des SREG 
	push TEMP	    	; Schreiben von  SREG  im Stack (KOPIE)
;-------
	rcall ENDE_SPEICHER_REFERENZ_PRUEFEN; ENDE erreicht?
	tst ENDE_SPEICHER					; ENDE erreicht?
	brne ADC_HANDLER_STOP				; ja=>Sprung	
	adiw ZL,1							; Zhler um eins erhhen	
; ADC einlesen				
	in TEMP1, ADCL 	; L		
	in TEMP2, ADCH 	; H		

; 10 => 8 bit			
	LSR temp2
	ROR temp1	
	LSR temp2
	ROR temp1	
	ST X+,temp1   	;DATEN SPEICHERN und Zeiger um 1 erhhen	
	rjmp ADC_HANDLER_ENDE	
; ADC STOP				
ADC_HANDLER_STOP:	
	clr temp			
	out ADCSRA, temp	; ADC STOP	

	rcall BERECHNUNG_GLEICHSPANNUNGSANTEIL
	rcall AUSGABE_SRAM_WERTE
	clr ADC_RUN			

ADC_HANDLER_ENDE:	
;-------
	pop	TEMP			; LESEN von SREG vom STACK (KOPIE)		
	out	SREG,TEMP		; Wiederherstellen von SREG       		
	pop temp3      		; Wiederherstellen von TEMP3    		
	pop temp2      		; Wiederherstellen von TEMP2     		
	pop temp1      		; Wiederherstellen von TEMP1     		
	pop temp       		; Wiederherstellen von TEMP      		
reti
;####################################################################################
;####################################################################################
;####################################################################################
AUSGABE_SRAM_WERTE:
;----------------	
	ldi XL, low(adr_daten_SPEICHER)		; X-DATEN-Initialisierung		
	ldi XH,high(adr_daten_SPEICHER)		; X-DATEN-Initialisierung		
	clr ZL								; Z-ZHLER-RESET (Anzahl WERTE)	
	clr ZH								; Z-ZHLER-RESET (Anzahl WERTE)	
	clr ENDE_SPEICHER					; Ende erreicht 0=nein 1=ja		
;----------------
	ldi temp1, '#'
    rcall serout                      	; SENDE_ROUTINE
	ldi temp1, '#'
    rcall serout                      	; SENDE_ROUTINE
	ldi temp1, '#'
    rcall serout                      	; SENDE_ROUTINE
	ldi temp1, '#'
    rcall serout                      	; SENDE_ROUTINE
;----------------
	ldi temp1, low(ANZAHL_DATENPUNKTE)	;(Anzahl Werte) senden 
    rcall serout                      	; SENDE_ROUTINE
	ldi temp1, high(ANZAHL_DATENPUNKTE) ;(Anzahl Werte) senden 
    rcall serout                      	; SENDE_ROUTINE


;----------------
AUSGABE_SRAM_WERTE_w:
	rcall ENDE_SPEICHER_REFERENZ_PRUEFEN; ENDE erreicht?
	tst ENDE_SPEICHER					; ENDE erreicht?
	brne AUSGABE_SRAM_WERTE_ENDE		; ja=>Sprung	

	adiw ZL,1							; Zhler um eins erhhen
	LD temp1, X+   						; DATEN aus SPEICHER laden und Zeiger um 1 erhhen	
    rcall serout                      	; SENDE_ROUTINE
	rjmp AUSGABE_SRAM_WERTE_w
AUSGABE_SRAM_WERTE_ENDE:

ret
;####################################################################################
;####################################################################################
;####################################################################################
; 	INPUT: temp1
serout:
    sbis UCSRA,UDRE                   ; Warten bis UDR fr das nchste
                                      ; Byte bereit ist
    rjmp serout
    out UDR, temp1

ret                               
;###################################################################
;###################################################################
;###################################################################
; von allen gespeicherten Daten 						
; wird der Wert 127 (Gleichspannungsanteil) subtrahiert
BERECHNUNG_GLEICHSPANNUNGSANTEIL:

	ldi XL, low(adr_daten_SPEICHER)		; X-DATEN-Initialisierung
	ldi XH,high(adr_daten_SPEICHER)		; X-DATEN_Initialisierung
	clr ZL								; Z-ZHLER-RESET (Anzahl WERTE)
	clr ZH								; Z-ZHLER-RESET (Anzahl WERTE)
	clr temp		
	clr ENDE_SPEICHER	

BERECHNUNG_GLEICHSPANNUNGSANTEIL_w:

	rcall ENDE_SPEICHER_REFERENZ_PRUEFEN
	tst ENDE_SPEICHER					; ENDE Speicher ereicht?
	brne BERECHNUNG_GLEICHSPANNUNGSANTEIL_ENDE; ja			
		
	adiw ZL,1				; Zhler um 1 erhhen			
	LD temp, X   			; DATEN aus SPEICHER laden		
	subi temp,127			; 127 Subtraktion				
	ST X+,temp 				; Daten im SPEICHER schreiben und Zeiger um 1 erhhen
	
	rjmp BERECHNUNG_GLEICHSPANNUNGSANTEIL_w

BERECHNUNG_GLEICHSPANNUNGSANTEIL_ENDE:

ret
;########################################################################
;########################################################################
;########################################################################
; INPUT: 	ZL+ZH						
; OUTPUT:	ENDE_SPEICHER  0=nein 1=ja	
ENDE_SPEICHER_REFERENZ_PRUEFEN:
	cpi ZH,high(ANZAHL_DATENPUNKTE)			; vergleich 
	brlo ENDE_SPEICHER_REFERENZ_PRUEFEN_ENDE; kleiner =>SPRUNG

	cpi ZL,low (ANZAHL_DATENPUNKTE)			; vergleich 
	brlo ENDE_SPEICHER_REFERENZ_PRUEFEN_ENDE; kleiner =>SPRUNG
	brne ENDE_SPEICHER_REFERENZ_PRUEFEN_ENDE; gleich  =>SPRUNG
	ldi temp,1
	mov ENDE_SPEICHER,temp
	ENDE_SPEICHER_REFERENZ_PRUEFEN_ENDE:
ret
;######################################################################## 
;######################################################################## 
;######################################################################## 
;	Warteschleife 		
wait:	
			ldi	temp3,20			
			clr	temp2			
			clr	temp1		
wait_:	
			dec	temp1		
			brne wait_		
			dec	temp2		
			brne wait_			
			dec	temp3			
			brne wait_			
ret		
;######################################################################## 
;######################################################################## 
;######################################################################## 
TEXT_SIMULATION_2: ; RECHTECK 154 WERTE					
.db 128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128
.db 128,128,128,128,128,128,128,126
.db 126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126,126
.db 126,126,126,126,126,126,126,126,0

